/*
 * MZTalk - a small and simple talk program
 *
 * Copyright 2004 Marcello Zaniboni
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */

package marcellozaniboni;

import java.io.BufferedReader;
import java.io.InputStreamReader;
import java.io.OutputStreamWriter;
import java.io.PrintWriter;
import java.net.InetAddress;
import java.net.ServerSocket;
import java.net.Socket;
import java.net.URL;

public class TalkManager {

	public static final String APP_TITLE = "MZ Talk - version 1.00 beta";
	public static final int RECEIVER_REFRESH = 500;
	public static final int SENDER_REFRESH = 500;
	public static final int SHUTDOWN_TIME = 2000;
	public static final String CLOSE_MESSAGE = "__quit__";
	public static final String HELP_FILE = "/resources/help.html";
	public static final String LICENSE_FILE = "/resources/gpl.html";
	public static final String APP_ICON = "/resources/MZTalkIcon.gif";
	
	private static final int PORT = 8765;
	private static final String SOUND_ALERT = "/resources/connection.wav";
	private String key = null;
	private String signature = null;
	private TalkKeyGen keyManager = null;
    private ITalkUI gui = null;
	private boolean servermode = false;
	private String remoteserver = null;
	private BufferedReader bufferedreader = null;
	private PrintWriter printwriter = null;
	private ServerSocket serversocket = null;
	private Socket socket = null;

    TalkManager (ITalkUI ui, boolean srv, String host) {
        gui = ui;
        servermode = srv;
        remoteserver = host;
    }

	public void init() throws Throwable {
		keyManager = new TalkKeyGen();
		signature = System.getProperty("user.name");
		if (signature != null) signature = "[" + signature.toUpperCase() + "] ";
		boolean connected = false;
		try {
			// get the connection
			try {
				if (servermode) {
					System.out.println("running in server mode");
					serversocket = new ServerSocket(PORT);
					socket = serversocket.accept();
					bufferedreader = new BufferedReader(new InputStreamReader(socket.getInputStream()));
					printwriter = new PrintWriter(new OutputStreamWriter(socket.getOutputStream()));
					key = keyManager.getNewKey();
					System.out.println("key generated");
				} else {
					System.out.println("using server \"" + remoteserver + "\"");
					InetAddress ia = InetAddress.getByName(remoteserver);
					socket = new Socket(ia, PORT);
					bufferedreader = new BufferedReader(new InputStreamReader(socket.getInputStream()));
					printwriter = new PrintWriter(new OutputStreamWriter(socket.getOutputStream()));
				}
				connected = true;
			} catch (Throwable netError) {
				gui.reportFatalError("PROBLEMS WHILE TRYING TO CONNECT:\n\""
					+ netError.getClass().getName() + " - "
					+ netError.getLocalizedMessage() + "\"");
				netError.printStackTrace(System.err);
			}
			if (connected) {
				System.out.println("connection info: " + socket);
				gui.setEnabled();
				gui.putMessage("Connected to:\n " + socket.toString() + "\n");

				// key excange
				if (servermode) {
					printwriter.println(key);
					printwriter.flush();
					System.out.println("key sent");
					alertWithSound();
				} else {
					key = bufferedreader.readLine().trim();
					System.out.println("key received");				
				}

				// run the threads
				TalkSender sender = new TalkSender(gui, printwriter, this);
				sender.start();
				TalkReceiver receiver = new TalkReceiver(gui, bufferedreader, this);
				receiver.start();
			} else {
				gui.putMessage("   CONNECTION FAILED.\n   PLEASE RESTART THE APPLICATION.");
			}

		} catch (Throwable t) {
			t.printStackTrace(System.err);
			throw t;
		}
	}

	private void alertWithSound() {
		try {
			URL url = null;
			try {
				url = this.getClass().getResource(SOUND_ALERT); 
			} catch (Throwable t) {
				System.err.println("problems opening resource " + SOUND_ALERT);
			}
			if (url != null) {
				java.applet.AudioClip clip = java.applet.Applet.newAudioClip(url);
				clip.play();
			} else {
				System.err.println("resource " + SOUND_ALERT + " not found");
			}
		} catch (Throwable t) {
			t.printStackTrace(System.err);
		}
	}

	public synchronized void destroy() {
		try {
			if (serversocket != null) {
				serversocket.close();
				serversocket = null;
			}
			if (socket != null) {
				socket.close();
				socket = null;
			}			
		} catch (Throwable t) { }
	}

	public final String getKey() { return key; }
	public final TalkKeyGen getKeyManager() { return keyManager; }
	public boolean isServermode() { return servermode; }
	public String getSignature() { return signature; }

}

